<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use Twig_Environment;

/**
 * Environment
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Twig
 * @since   Infomaker\Everyware\Twig\Environment 0.1
 */
class Environment extends Twig_Environment
{

    /**
     * Loads a specific template and compiles it. If cache is on, save the compiled class in cac3he to be evaluated
     * later on.
     *
     * @param string $name
     * @param int|null $index
     *
     * @return \Twig_TemplateInterface
     * @throws \Twig_Error_Loader
     * @throws \Twig_Error_Runtime
     * @throws \Twig_Error_Syntax
     */
    public function loadTemplate($name, $index = null): \Twig_TemplateInterface
    {
        $cls = $mainCls = $this->getTemplateClass($name);

        if (isset($this->loadedTemplates[$cls])) {
            return $this->loadedTemplates[$cls];
        }

        if (!class_exists($cls, false)) {
            if ($this->cache instanceof objectCacheSystem) {
                // Fetch template from cache and load
                $this->cache->load($this->cache->generateKey($name, $mainCls));
            }
        }

        return parent::loadTemplate($name, $index);
    }
}
