<?php declare(strict_types=1);
/**
 * Twig Extension Class: for handling the filesystem of Twig
 */

namespace Infomaker\Everyware\Twig;

use RecursiveDirectoryIterator;
use RecursiveFilterIterator;
use RecursiveIterator;
use RecursiveIteratorIterator;
use Twig_Error;
use Twig_Loader_Chain;
use Twig_Loader_Filesystem;
use Twig_LoaderInterface;

/**
 * Every_Loader
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Twig
 * @since   Infomaker\Everyware\Twig\Every_Loader 0.1
 */
class EveryLoader extends Twig_Loader_Filesystem
{

    /**
     * Adds the folders containing the files to be loaded
     *
     * @param string $folder_path
     * @param string $namespace
     * @param array $exclude
     *
     * @throws Twig_Error
     * @return void
     */
    public function addPathAndSubfolders(
        string $folder_path,
        string $namespace = Twig_Loader_Filesystem::MAIN_NAMESPACE,
        array $exclude = []
    ): void {
        if (!file_exists($folder_path)) {
            throw new Twig_Error($folder_path . " does not exist");
        }

        $dir = new RecursiveDirectoryIterator($folder_path);
        $filtered = new DirFilter($dir, $exclude);

        foreach (new RecursiveIteratorIterator($filtered) as $d) {
            $this->addPath($d, $namespace);
        }
    }

    /**
     * No documentation yet
     *
     * @todo  Add documentation
     *
     * @param array $array
     *
     * @return Twig_Loader_Chain
     */
    public function getChain(array $array): \Twig_Loader_Chain
    {
        return new Twig_Loader_Chain($array);
    }
}
