<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use Infomaker\Everyware\Support\Storage\Cache;
use SplFileInfo;
use Twig_Error;
use Twig_Loader_Chain;
use Twig_Loader_Filesystem;

/**
 * FilesystemLoader
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Twig
 * @since   Infomaker\Everyware\Twig\FilesystemLoader 0.1
 */
class FilesystemLoader extends Twig_Loader_Filesystem
{

    /**
     * Time in seconds to cache the output if cache is set to true
     *
     * @var int
     */
    private $cache_time = 86400;

    /**
     * Add suffix so we don't have to delete it in deploy every time
     *
     * @var string
     */
    private $cache_key = 'everytwig_paths_cache_' . GIT_COMMIT;

    /**
     * Adds the folders containing the files to be loaded
     *
     * @param string $folder_path
     * @param string $namespace
     * @param array $exclude
     *
     * @throws Twig_Error
     * @return void
     */
    public function addPathAndSubfolders(
        string $folder_path,
        string $namespace = Twig_Loader_Filesystem::MAIN_NAMESPACE,
        array $exclude = []
    ): void {
        if (!file_exists($folder_path)) {
            throw new Twig_Error("{$folder_path} does not exist");
        }
        $cache_key = $this->getPathCacheKey($namespace);
        $paths = Cache::get($cache_key, []);

        if (empty($paths)) {
            $dir = PathFilter::setupFromPath($folder_path, $exclude);
            $paths = [];
            foreach ($dir->filtered() as $d) {
                if ($d instanceof SplFileInfo) {
                    $paths[] = str_replace('//', '/', $d->getPath());
                }
            }

            $paths = array_unique($paths);
            Cache::set($cache_key, $paths, $this->cache_time);
        }

        foreach ($paths as $path) {
            $this->addPath($path, $namespace);
        }
    }

    /**
     *
     *
     * @param $namespace
     *
     * @return string
     */
    private function getPathCacheKey(string $namespace): string
    {
        return "{$this->cache_key}_{$namespace}";
    }

    /**
     * No documentation yet
     *
     * @todo  Add documentation
     *
     * @param array $array
     *
     * @return Twig_Loader_Chain
     */
    public function getChain(array $array): Twig_Loader_Chain
    {
        return new Twig_Loader_Chain($array);
    }

    /**
     * Set cache time in seconds
     *
     * @param $time
     *
     * @return void
     */
    public function setCacheTime(int $time): void
    {
        $this->cache_time = $time;
    }
}
