<?php declare(strict_types=1);

namespace Infomaker\Everyware\Twig;

use Infomaker\Everyware\Support\Storage\Cache;
use Twig_Cache_Filesystem;
use Twig_CacheInterface;

/**
 * CacheLoader
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Twig
 * @since   Infomaker\Everyware\Twig\CacheLoader 0.1
 */
class ObjectCacheSystem extends Twig_Cache_Filesystem implements Twig_CacheInterface
{

    /**
     * Add suffix so we don't have to delete it in deploy every time
     *
     * @var string
     */
    private static $cache_key = '';

    /**
     * Time in seconds to cache the output
     *
     * @var int
     */
    private static $cache_time = 360;

    /**
     * Static function for initialization
     *
     * @param string $cache_key_prefix
     * @param int $cache_time
     *
     * @return static
     */
    public static function setup(string $cache_key_prefix, int $cache_time = 60): ObjectCacheSystem
    {
        static::$cache_key = $cache_key_prefix;
        static::$cache_time = $cache_time;

        return new static(static::$cache_key);
    }

    /**
     * Writes the compiled template to cache.
     *
     * @param string $key The cache key
     * @param string $content The template representation as a PHP class
     *
     * @return void
     */
    public function write($key, $content): void
    {
        Cache::set($this->getCacheKey($key), serialize($content), static::$cache_time);
    }

    /**
     * Create a cache key combined with the key prefix provided
     * during setup and a hashed version of the native key provided by Twig
     *
     * @param $key
     *
     * @return string
     */
    protected function getCacheKey(string $key): string
    {
        return static::$cache_key . '_' . md5($key);
    }

    /**
     * Loads a template from the cache.
     *
     * @param string $key The cache key
     *
     * @return void
     */
    public function load($key): void
    {
        if (false !== ($content = Cache::get($this->getCacheKey($key)))) {
            eval('?>' . unserialize($content));
        }
    }
}
